﻿package com  {  

	import flash.events.*;
    import flash.display.MovieClip;
	
	import com.SongAlbum;
	import com.PlaybackInterface;
	import com.AlbumState;
	import com.AlbumMenu;
	import com.Scrollbar;
	import com.LyricsHandler;
	import com.TrackListHandler;
	
	import com.greensock.*;
	import com.greensock.easing.*;	
	
	public class Main extends MovieClip {  
    	
		//Allows our Main instance to be accessed (via a getter) from inside other classes (specifically, TweenMax)
		private static var _instance:Main;
		public static function get instance():Main { return _instance; }
		
		/* Create a top-level dispatcher */
		public var dispatcher = new EventDispatcher();
		
		/* By keeping a reference to the album, we can call any of SongAlbum's methods for special cases
		*  Theoretically, we could instead just pass a new SongAlbum object to the PlaybackInterface, but
		*  we would lose the ability to control it outside that Interface */
		public var album:SongAlbum;
		
		public var menu:AlbumMenu;
		
		public var states:Object;
		public var currentState:AlbumState;
		public var previousState:AlbumState;
		
		//public var playbackControls:Object;
		public var playback:PlaybackInterface;

		/**
		 * Causes init() to be executed when the Main object is added to the stage
		 *
		 * Code is moved to init() to avoid any runtime errors when later referencing onstage MovieClips
		 * by their instance names (we don't want the code to load before the stage initializes)
		 */
		public function Main() { 
			_instance = this;
			addEventListener(Event.ADDED_TO_STAGE, init, false, 0, true);
		}  
		
		/**
		 * Initializes some variables and attaches the necessary eventListeners to set
		 * our prototype in motion. The rest will be handled by the eventListeners' methods
		 *
		 * @param e Event object passed from the EventListener inside Main()
		 */
		public function init(e:Event):void {
					
			removeEventListener(Event.ADDED_TO_STAGE, init);		
					
			//Adding one listener to the Stage will cause the drag event to take priority over scroll events		
			var dragHandles:Array = [bg_Artist1,bg_Home1,bg_Lyrics1,bg_Tracks1];
			for each(var handle:MovieClip in dragHandles) {
				handle.addEventListener(MouseEvent.MOUSE_DOWN, function(e:MouseEvent):void {
					stage.nativeWindow.startMove();
				});
			}		
					
			/* Instantiate the SongAlbum */
			album = new SongAlbum(dispatcher);
						
			/* Instantiate the AlbumMenu */
			menu = new AlbumMenu(dispatcher, m_item_Home1,m_item_Tracks1,m_item_Lyrics1,m_item_Artist1,m_indicator1);
			
			//menu.addEventListener(AlbumMenu.MENU_BUTTON_CLICKED, menuButtonClicked);
			//When a state transition completes, allow for addition menu input – WHY WON'T THIS WORK?!
			//Remember that in AlbumMenu, you've commented out the code that sets acceptingInput to 'false'
			//menu.addEventListener(AlbumState.TRANSITION_COMPLETE, function(e:Event){ menu.acceptingInput = true; trace("ACCEPTING INPUT"); });
			
			dispatcher.addEventListener(AlbumMenu.MENU_BUTTON_CLICKED, menuButtonClicked);
			dispatcher.addEventListener(AlbumState.TRANSITION_COMPLETE, function(e) { menu.acceptingInput = true; });
			
			/* Instantiate the PlaybackInterface */
			playback = new PlaybackInterface(album, playbackScrubber1, playbackControls1, playbackHitbox1);
					
			/* Instantiate the TrackListHandler */
			var trackListHandler:TrackListHandler = new TrackListHandler(dispatcher, album,content_Tracks1,playback);
			album.addTrackListHandler(trackListHandler);		
					
			/* Instantiate each AlbumState object */
			states = { home:   new AlbumState(dispatcher, 'home',   content_Home1,   ctx_Home1,   bg_Home1),
					   tracks: new AlbumState(dispatcher, 'tracks', content_Tracks1, ctx_Tracks1, bg_Tracks1),
					   lyrics: new AlbumState(dispatcher, 'lyrics', content_Lyrics1, ctx_Lyrics1, bg_Lyrics1),
					   artist: new AlbumState(dispatcher, 'artist', content_Artist1, ctx_Artist1, bg_Artist1) };

			/* Set the initial state to 'tracks' */
			currentState = states.home;

			/* Attach any animation completion handlers */
			
			dispatcher.addEventListener(AlbumState.CONTENT_COMPLETE, function(e) { currentState.toggleContent(); } );
			dispatcher.addEventListener(AlbumState.CONTEXT_COMPLETE, function(e) { currentState.toggleContext(); } );
			/*
			for each(var s:AlbumState in states) {
				s.addEventListener(AlbumState.CONTENT_COMPLETE,function(e:Event){ currentState.toggleContent(); });
				s.addEventListener(AlbumState.CONTEXT_COMPLETE,function(e:Event){ currentState.toggleContext(); });
			}
			*/
			/* Instantiate our LyricsHandler */
			var lyricsHandler:LyricsHandler = new LyricsHandler(dispatcher, album, content_Lyrics1);
			
			//Set up our starting state (change this to 'HOME' later ...)
			setState(states.home);

		}
		
		/**
		 * Fires when the AlbumMenu dispatches a MENU_BUTTON_CLICKED event.
		 * Calls setState() depending on the context of the click
		 */
		private function menuButtonClicked(e):void {
			
			//Determine the targeted state, based on e.currentTarget
			var targetState:AlbumState;
			for each(var s:AlbumState in states)
				if(s.handle == menu.currentButton.handle)
					targetState = s;	
						
			//Call setState() when appropriate, passing it the target state object
			if(currentState != null) {
				if(targetState.handle != currentState.handle)
					setState(targetState);
			}
			else setState(targetState);
			
		}

		/* We're keeping the setState() function separate because it may be called
		   by something other than a mouse click */
		public function setState(targetState:AlbumState):void {
			
			//Update previousState and currentState
			previousState = currentState;
			currentState = targetState;
			trace("Switching to state '" + currentState.handle + "'.");
			
			/* For each state transition, we want to do a few things ... */
			
			//1. Update the 'isCurrent' property for each state
			for each(var s:AlbumState in states)
				s.isCurrent = false;
			currentState.isCurrent = true;
			
			//2. Hide the previous content and reveal the target state's content
			//   – Dispatch an event (listened for by Main) that triggers the new content to fade in
			previousState.toggleContent();

			//3. Transition to the target state's context
			//   – Dispatch an event (listened for by Main) that triggers the new context to be shown
			previousState.toggleContext();
			
			//4. Transition to the target state's background
			previousState.toggleBackground();
			currentState.toggleBackground();
		
		}
	}
}