﻿package com  {
	
	import flash.display.MovieClip;
	import flash.events.*;
	
	import com.SongAlbum;
	import com.PlaybackInterface;
	
	import com.greensock.TweenLite;
	import com.greensock.easing.*;
	
	public class TrackListHandler {

		private var _dispatcher:EventDispatcher;

		private var _album:SongAlbum;
		private var _trackList:MovieClip;
		private var _trackItems:Vector.<TrackItem> = new Vector.<TrackItem>(0,false); //Empty, but of unfixed length
		private var _playbackInterface:PlaybackInterface;

		private var _currentTrackItem;

		public function TrackListHandler(dispatcher_:EventDispatcher, album_:SongAlbum, trackList_:MovieClip, playbackInterface:PlaybackInterface ) {
			
			_album = album_;
			_trackList = trackList_;
			_playbackInterface = playbackInterface;
		
			//Add an event listener to _dispacher
			_dispatcher = dispatcher_;
			//When our album fires a "SONG_CHANGE" event, we'll need to update _currentTrackItem (and list opacities)
			_dispatcher.addEventListener(SongAlbum.SONG_CHANGE, updateTrackList);
		
		}
		
		private function updateTrackList(e:Event):void {
			
			var data = _album.getCurrentSongData();
			var newTrackIndex = data.trackNumber - 1; // [1,14] -> [0,13]
			for each(var trackItem:TrackItem in _trackItems) { trackItem.alpha = 0.5; }
			_trackItems[newTrackIndex].alpha = 1.0;

		}
		
		public function addTrack(trackData:Array):void {
			
			var trackTitle:String = trackData[0];
			var trackNumber:String = trackData[1].toString();
						
			//Create a new TrackItem and add it to our _trackItems group
			var trackItem:TrackItem = new TrackItem();
			_trackItems.push(trackItem);
			trackItem.trackTitle.text = trackTitle;
			trackItem.trackNumberTextbox.trackNumber.text = trackNumber;
			
			//Fill in track info
			var trackItemAsMC:MovieClip = trackItem as MovieClip;
			trackItemAsMC.titleHandle = trackTitle;  //Custom properties for later ...
			trackItemAsMC.numberHandle = trackNumber;
			
			//Add the new TrackItem to the 'track's content container
			trackItem.y = -32 + trackItem.height*int(trackNumber);
			trackItem.alpha = 0.5;
			trackItem.buttonMode = true;
			_trackList.addChild(trackItem);
			
			//Add an event listeners to the TrackItem clip
			trackItem.addEventListener(MouseEvent.MOUSE_OVER, trackItemMouseover);
			trackItem.addEventListener(MouseEvent.MOUSE_OUT, trackItemMouseover);
			trackItem.addEventListener(MouseEvent.CLICK, trackItemOnClick);
			
		}

		function trackItemMouseover(e:MouseEvent):void {
			var toggleAlpha:Object = { mouseOver: 1.0, mouseOut: 0.5 };
			var targetTrackItem = e.currentTarget as MovieClip;
			//Mouseover and mouseout don't affect current track item (current song)
			if(targetTrackItem != _currentTrackItem as MovieClip) {
				TweenLite.to(targetTrackItem,0.5,{ alpha:toggleAlpha[e.type], ease:Quint.easeOut });
			}
		}
		
		function trackItemOnClick(e:MouseEvent):void {
			//Let the PlaybackInterface know that it's playing
			//Make sure that the play/pause button is set to 'play'
			_playbackInterface.setIsPlaying(true);
			_playbackInterface.togglePlay.gotoAndStop("pause");
			//Update the new (current) track's appearance
			if(_currentTrackItem != null) _currentTrackItem.alpha = 0.5;
			//We include 'parent' because the target will always be a child (title, number, etc)
			_currentTrackItem = e.target.parent as MovieClip; 
			_currentTrackItem.alpha = 1.0;
			//Have the SongAlbum play the selected track
			_album.pause();
			_album.play(_currentTrackItem.titleHandle);
		}

	}
	
}
